{% extends "iop4admin/singleobj.html" %}

{% block content %}

<h2>Fields</h2>

<table>
    <thead>
      <tr>
        <th></th>
        {% for k, v in fields_and_values.items %}
          {% if v != "None" %}
            <th>{{ k }}</th>
          {% endif %}
        {% endfor %}
      </tr>
    </thead>
    <tbody>
      <tr>
        <th></th>
        {% for k, v in fields_and_values.items %}
          {% if v != "None" %}
            <td>{{ v }}</td>
          {% endif %}
        {% endfor %}
      </tr>
    </tbody>
</table>

<h2>Comment</h2>

{{ object.comment_html | safe }}


<h2>Finding chart</h2>
<img src="data:image/png;base64,{{ finding_chart_b64 }}"</img>

{{ block.super }}
{% endblock %}